==> gtphoto2-framework-0.1.tar.gz: gtphoto2/includes/paginate_functions.php <==
<?php

/*

    GTPhoto2 - A multi user photo album
    Copyright (C) 2004 Chris Kloosterman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Author's Email Address:
        kloostec@gavintech.com
    Paper Mail:
        5945 Cassino Road
        Duncan, BC, V9L 4G5, Canada

*/

# This is a crazy function ;)
# pass it a 2-D array and it'll draw out a table
# with sort functionality

# Notes:
# assumes values already cleaned for *display*

# parameters:
# $array - the 2D array to be sorted and displayed
#   - If the column is a number, the name will not be displayed as a heading (and it will not be sortable, etc)
#   - Do NOT use 0 as a column header, as the pagination function will break
# $default_sort_column - the column to sort by if nothing else is selected
# $default_sort_direction - the direction to sort by if nothing else is selected
#   - possible values: SORT_ASC, SORT_DESC
# $number_per_page - the number of elements to display per page
#   - if passed 0, the function will display all elements
# $full_width_column - if you want a column to take up as much room as possible, set it here
#   - if blank, 
# $carry_vars - the get string (with preceeding &) to append to links to carry over variables

function paginate(&$array, $default_sort_column, $default_sort_direction, $number_per_page = 0, $full_width_column = "", $carry_vars="") {

	# check to make sure we've been passed a nice array
	if (!(is_array($array))) {
		puke ("\$array not an array!");
	} else {
		if (!(is_array($array[0]))) {
			puke ("\$array not a proper 2D array!");
		}
	}

	$sort_order = array();
	$original_order = array();

	# figure out the order in which to sort
	if ($_REQUEST["paginate_sort"]) {
		array_push($sort_order,$_REQUEST["paginate_sort"]);
	} else {
		array_push($sort_order, $default_sort_column);
	}
	foreach ($array[0] as $key => $value) {
		if (array_search($key, $sort_order) === false) {
			array_push($sort_order, $key);
		}
		array_push($original_order, $key);
	}
	
	if (DEBUG) {
		echo ("I'm going to sort in this order: ");
		print_r($sort_order);
		echo ("<br /><br />");
	}

	# reorg the array into something useful
	foreach ($sort_order as $key => $value) {
		$sort_array[$value] = array();
		foreach ($array as $key2 => $value2) {
			$sort_array[$value][$key2] = $array[$key2][$value];
		}
	}

	if (DEBUG) {
		echo ("Array before sorting: <pre>");
		print_r ($sort_array);
		echo ("</pre>");
	}

	# figure out the direction in which to sort
	if ($_REQUEST["paginate_direction"]) {
		$sort_direction = $_REQUEST["paginate_direction"];
	} else {
		$sort_direction = $default_sort_direction;
	}

	if (DEBUG) {
		echo ("I'm going to sort in this direction: $sort_direction<br /><br />");
	}

	# figure out what page we're on
	if ($_REQUEST["paginate_page"]) {
		$sort_page = $_REQUEST["paginate_page"];
	} else {
		$sort_page = 1;
	}

	# figure out how many elements to display
	if ($_REQUEST["paginate_count"]) {
		$sort_count = $_REQUEST["paginate_count"];
	} else {
		$sort_count = $number_per_page;
	}

	# build the array_multisort command
	$command = "array_multisort(";
	$col = array_shift($sort_order);
	$command .= "\$sort_array[\"$col\"], $sort_direction";
	foreach ($sort_order as $key => $value) {
		$command .= ", \$sort_array[\"$value\"]";
	}
	$command .= ");";

	if (DEBUG) {
		echo ("I'm going to run the following command: '$command' <br /><br />");
	}
	
	# run the sort
	eval($command);

	if (DEBUG) {
		echo ("Sorted array: <pre>");
		print_r ($sort_array);
		echo ("</pre>");
	}
	
	# figure out the rest of the pagination stuff
	if ($sort_count != 0) {
		foreach ($sort_array as $key => $value) {
			$sort_array[$key] = array_slice($sort_array[$key], (($sort_page - 1) * $sort_count), $sort_count);
		}
	}

	if (DEBUG) {
		echo ("Sliced array: <pre>");
		print_r ($sort_array);
		echo ("</pre>");
	}
	
	# print out a table

	# figure out the width of each column
	if ($full_width_column != "") {
		$col_width = 0;
	} else {
		$col_width = 100/count($sort_array);
	}
	
	# generate the navigation code if necessary
	if ($sort_count != 0) {
		$nav = "<table width=\"100%\">\n<tr>\n<td width=\"33%\">&nbsp;</td>\n<td width=\"34%\" align=\"center\">";
		$nav .= "<table width=\"300\">\n<tr>\n";
		$nav .= "<td width=\"100\" align=\"left\">";
		if ($sort_page != 1) {
			$nav .= "<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=$col&paginate_page=".($sort_page-1)."&paginate_count=$sort_count&paginate_direction=$sort_direction".$carry_vars."\">&lt; Prev Page</a>\n</td>\n";
		} else {
			$nav .= "&nbsp;\n";
		}
		$nav .= "</td><td width=\"100\" align=\"center\">Page $sort_page/".ceil(count($array)/$sort_count)."</td><td width=\"100\" align=\"right\">";
		if (($sort_page * $sort_count) < (count($array))) {
			$nav .= "<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=$col&paginate_page=".($sort_page+1)."&paginate_count=$sort_count&paginate_direction=$sort_direction".$carry_vars."\">Next Page &gt;</a>\n</td>\n";
		} else {
			$nav .= "&nbsp;\n";
		}
		$nav .= "</td></table>";
		$nav .= "</td>\n<td width=\"33%\" align=\"right\">";
		$allowed_number_per_page = array(5,10,25,100);
		$nav .= "<p>Show ";
		foreach ($allowed_number_per_page as $value) {
			$nav .= "<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=".$col."&paginate_count=$value&paginate_direction=".$default_sort_direction.$carry_vars."\">$value</a> ";
		}
		$nav .= "per page</p>";
		$nav .= "</td>\n</tr>\n</table>";
	}
	
	# show navigation
	echo $nav;
	
	echo ("<table width=\"100%\" cellspacing=\"0\" border=\"1\" cellpadding=\"2\">\n");
	$keys = array_keys($array[0]);
	echo ("<tr>\n");
	foreach ($keys as $value) {
		if ($value == $full_width_column) {
			$this_col_width = 100;
		} else {
			$this_col_width = $col_width;
		}
		if (is_numeric($value)) {
			echo ("<td width=\"$this_col_width%\" class=\"paginate_0\">&nbsp;</td>");
		} else {
			if ($value == $col) {
				if ($sort_direction == SORT_DESC) {
					echo ("<td width=\"$this_col_width%\" class=\"paginate_0\">\n<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=$col&paginate_page=$sort_page&paginate_count=$sort_count&paginate_direction=".SORT_ASC.$carry_vars."\">$value</a>&nbsp;v\n</td>\n");
				} else {
					echo ("<td width=\"$this_col_width%\" class=\"paginate_0\">\n<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=$col&paginate_page=$sort_page&paginate_count=$sort_count&paginate_direction=".SORT_DESC.$carry_vars."\">$value</a>&nbsp;^\n</td>\n");
				}
			} else {
				echo ("<td width=\"$this_col_width%\" class=\"paginate_0\">\n<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=".$value."&paginate_page=$sort_page&paginate_count=$sort_count&paginate_direction=".$default_sort_direction.$carry_vars."\">$value</a>\n</td>\n");
			}
		}
	}
	echo ("</tr>\n");
	$paginate_counter = 0;
	foreach ($sort_array[$keys[0]] as $key => $value) {
		$paginate_counter++;
		echo ("<tr>\n");
		foreach ($keys as $value2) {
			echo ("<td class=\"paginate_".($paginate_counter % 2)."\">\n".$sort_array[$value2][$key]."\n</td>\n");
		}
		echo ("</tr>\n");
	}
	echo ("</table>\n");
	
	# show navigation
	echo $nav;
}

?>

==> gtphoto2-framework-0.2.tar.gz: gtphoto2/includes/paginate_functions.php <==
<?php

/*

    GTPhoto2 - A multi user photo album
    Copyright (C) 2004 Chris Kloosterman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Author's Email Address:
        kloostec@gavintech.com
    Paper Mail:
        5945 Cassino Road
        Duncan, BC, V9L 4G5, Canada

*/

# This is a crazy function ;)
# pass it a 2-D array and it'll draw out a table
# with sort functionality

# Notes:
# assumes values already cleaned for *display*

# parameters:
# $array - the 2D array to be sorted and displayed
#   - If the column is a number, the name will not be displayed as a heading (and it will not be sortable, etc)
#   - Do NOT use 0 as a column header, as the pagination function will break
# $default_sort_column - the column to sort by if nothing else is selected
# $default_sort_direction - the direction to sort by if nothing else is selected
#   - possible values: SORT_ASC, SORT_DESC
# $number_per_page - the number of elements to display per page
#   - if passed 0, the function will display all elements
# $full_width_column - if you want a column to take up as much room as possible, set it here
#   - if blank, 
# $carry_vars - the get string (with preceeding &) to append to links to carry over variables

# feature that I didn't find at first:
#   Instead of just passing in text as elements of 2D array
#   (ex: $sort_array[0]["Date"] = "Some text")
#   You can pass in an array, such as the following
#     $sort_array[0]["Date"][0] = 21321
#     $sort_array[0]["Date"][1] = "Text representation"
#   I have set this up so that the array is sorted on the first element [0]
#   and the second element [1] is what is actually displayed.  Very useful ;)

function paginate(&$array, $default_sort_column, $default_sort_direction, $number_per_page = 0, $full_width_column = "", $carry_vars="") {

	# check to make sure we've been passed a nice array
	if (!(is_array($array))) {
		puke ("\$array not an array!");
	} else {
		if (!(is_array($array[0]))) {
			puke ("\$array not a proper 2D array!");
		}
	}

	$sort_order = array();
	$original_order = array();

	# figure out the order in which to sort
	if ($_REQUEST["paginate_sort"]) {
		array_push($sort_order,clean_form_display($_REQUEST["paginate_sort"]));
	} else {
		array_push($sort_order, $default_sort_column);
	}
	foreach ($array[0] as $key => $value) {
		if (array_search($key, $sort_order) === false) {
			array_push($sort_order, $key);
		}
		array_push($original_order, $key);
	}
	
	if (DEBUG) {
		echo ("I'm going to sort in this order: ");
		print_r($sort_order);
		echo ("<br /><br />");
	}

	# reorg the array into something useful
	foreach ($sort_order as $key => $value) {
		$sort_array[$value] = array();
		foreach ($array as $key2 => $value2) {
			$sort_array[$value][$key2] = $array[$key2][$value];
		}
	}

	if (DEBUG) {
		echo ("Array before sorting: <pre>");
		print_r ($sort_array);
		echo ("</pre>");
	}

	# figure out the direction in which to sort
	if ($_REQUEST["paginate_direction"]) {
		$sort_direction = $_REQUEST["paginate_direction"];
	} else {
		$sort_direction = $default_sort_direction;
	}

	if (DEBUG) {
		echo ("I'm going to sort in this direction: $sort_direction<br /><br />");
	}

	# figure out what page we're on
	if ($_REQUEST["paginate_page"]) {
		$sort_page = $_REQUEST["paginate_page"];
	} else {
		$sort_page = 1;
	}

	# figure out how many elements to display
	if ($_REQUEST["paginate_count"]) {
		$sort_count = $_REQUEST["paginate_count"];
	} else {
		$sort_count = $number_per_page;
	}

	# build the array_multisort command
	$command = "array_multisort(";
	$col = array_shift($sort_order);
	$command .= "\$sort_array[\"$col\"], $sort_direction";
	foreach ($sort_order as $key => $value) {
		$command .= ", \$sort_array[\"$value\"]";
	}
	$command .= ");";

	if (DEBUG) {
		echo ("I'm going to run the following command: '$command' <br /><br />");
	}
	
	# run the sort
	eval($command);

	if (DEBUG) {
		echo ("Sorted array: <pre>");
		print_r ($sort_array);
		echo ("</pre>");
	}
	
	# figure out the rest of the pagination stuff
	if ($sort_count != 0) {
		foreach ($sort_array as $key => $value) {
			$sort_array[$key] = array_slice($sort_array[$key], (($sort_page - 1) * $sort_count), $sort_count);
		}
	}

	if (DEBUG) {
		echo ("Sliced array: <pre>");
		print_r ($sort_array);
		echo ("</pre>");
	}
	
	# print out a table

	# figure out the width of each column
	if ($full_width_column != "") {
		$col_width = 0;
	} else {
		$col_width = 100/count($sort_array);
	}
	
	# generate the navigation code if necessary
	if ($sort_count != 0) {
		$nav = "<table width=\"100%\">\n<tr>\n<td width=\"33%\">&nbsp;</td>\n<td width=\"34%\" align=\"center\">";
		$nav .= "<table width=\"300\">\n<tr>\n";
		$nav .= "<td width=\"100\" align=\"left\">";
		if ($sort_page != 1) {
			$nav .= "<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=".urlencode($col)."&paginate_page=".($sort_page-1)."&paginate_count=$sort_count&paginate_direction=$sort_direction".$carry_vars."\">&lt; Prev Page</a>\n</td>\n";
		} else {
			$nav .= "&nbsp;\n";
		}
		$nav .= "</td><td width=\"100\" align=\"center\">Page $sort_page/".ceil(count($array)/$sort_count)."</td><td width=\"100\" align=\"right\">";
		if (($sort_page * $sort_count) < (count($array))) {
			$nav .= "<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=".urlencode($col)."&paginate_page=".($sort_page+1)."&paginate_count=$sort_count&paginate_direction=$sort_direction".$carry_vars."\">Next Page &gt;</a>\n</td>\n";
		} else {
			$nav .= "&nbsp;\n";
		}
		$nav .= "</td></table>";
		$nav .= "</td>\n<td width=\"33%\" align=\"right\">";
		$allowed_number_per_page = array(10,25,50,100,250);
		$nav .= "<p>Show ";
		foreach ($allowed_number_per_page as $value) {
			$nav .= "<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=".urlencode($col)."&paginate_count=$value&paginate_direction=".$default_sort_direction.$carry_vars."\">$value</a> ";
		}
		$nav .= "per page</p>";
		$nav .= "</td>\n</tr>\n</table>";
	}
	
	# show navigation
	echo $nav;
	
	echo ("<table width=\"100%\" cellspacing=\"0\" border=\"1\" cellpadding=\"2\" style=\"border-collapse: collapse\">\n");
	$keys = array_keys($array[0]);
	echo ("<tr>\n");
	foreach ($keys as $value) {
		if ($value == $full_width_column) {
			$this_col_width = 100;
		} else {
			$this_col_width = $col_width;
		}
		if (is_numeric($value)) {
			echo ("<td width=\"$this_col_width%\" class=\"paginate_0\">&nbsp;</td>");
		} else {
			if ($value == $col) {
				if ($sort_direction == SORT_DESC) {
					echo ("<td width=\"$this_col_width%\" class=\"paginate_0\">\n<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=".urlencode($col)."&paginate_page=$sort_page&paginate_count=$sort_count&paginate_direction=".SORT_ASC.$carry_vars."\">$value</a>&nbsp;v\n</td>\n");
				} else {
					echo ("<td width=\"$this_col_width%\" class=\"paginate_0\">\n<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=".urlencode($col)."&paginate_page=$sort_page&paginate_count=$sort_count&paginate_direction=".SORT_DESC.$carry_vars."\">$value</a>&nbsp;^\n</td>\n");
				}
			} else {
				echo ("<td width=\"$this_col_width%\" class=\"paginate_0\">\n<a href=\"".$_SERVER["SCRIPT_NAME"]."?next_page=".$_REQUEST["next_page"]."&amp;paginate_sort=".urlencode($value)."&paginate_page=$sort_page&paginate_count=$sort_count&paginate_direction=".$default_sort_direction.$carry_vars."\">$value</a>\n</td>\n");
			}
		}
	}
	echo ("</tr>\n");
	$paginate_counter = 0;
	foreach ($sort_array[$keys[0]] as $key => $value) {
		$paginate_counter++;
		echo ("<tr>\n");
		foreach ($keys as $value2) {
			if (is_array($sort_array[$value2][$key])) {
				$text = $sort_array[$value2][$key][1];
			} else {
				$text = $sort_array[$value2][$key];
			}
            
            // add an nbsp if the content of the cell is
            // otherwise empty (so it doesn't look like ass when
            // it gets rendered)
            if(!$text){
                $text = "&nbsp;" ;
            }
			echo ("<td class=\"paginate_".($paginate_counter % 2)."\">\n".$text."\n</td>\n");

		}
		echo ("</tr>\n");
	}
	echo ("</table>\n");
	
	# show navigation
	echo $nav;
}

/*
 * This function generates a carry string for use
 * in the paginate function.  Pass it an array
 * of the REQUEST variables you'd like to carry over
 *
 * This function uses the "get_string_recursive" function
 * in sidebar.php heavily.
 */

function generate_carry($array,$brackets="") {
	if (!(is_array($array))) {
		return "";
	}
	$array2 = array();
	foreach ($array as $value) {
		$array2[$value] = $_REQUEST[$value];
	}
        $get_string = get_string_recursive($array2,"");
	return "&".$get_string;
}

?>
